# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from core.BaseItem import *
from revkit import circuit, line_reduction

from helpers.RevKitHelper import *

from ui.DesignerWidget import DesignerWidget
from ui.LineReduction import Ui_LineReduction

class LineReduction( DesignerWidget ):
    def __init__( self, parent = None ):
        DesignerWidget.__init__( self, Ui_LineReduction, parent )

class LineReductionItem( BaseItem ):
    """This item provides the circuit line reduction method. After the item has been processed, the enlarged item reports the run-time needed to perform the optimization."""
    description = "Line Reduction"
    custom_size = QSize( 240, 140 )
    iconname = "format-indent-less"

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )
        self.requires( [ "Circuit" ] )
        self.provides( [ "Circuit" ] )

    def onCreate( self ):
        self.setText( "Line Reduction" )
        self.setState( self.CONFIGURED )

        self.widget = self.mainWidget( LineReduction() )

    def executeEvent( self, inputs ):
        circ = circuit()
        res = line_reduction( circ, inputs[0],
                              max_window_lines = self.widget.max_window_lines.value(),
                              max_grow_up_window_lines = self.widget.max_grow_up_window_lines.value(),
                              window_variables_threshold = self.widget.window_variables_threshold.value() )
        if type( res ) == dict:
            self.widget.runtime.setText( "%.2f s" % res['runtime'] )
            circuit_add_runtime( circ, res['runtime'] )
        else:
            pass
        return [ circ ]

    def setMaxwindowlines( self, v ):
        self.widget.max_window_lines.setValue( int( str( v ) ) )

    def setMaxgrowupwindowlines( self, v ):
        self.widget.max_grow_up_window_lines.setValue( int( str( v ) ) )

    def setWindowvariablesthreshold( self, v ):
        self.widget.window_variables_threshold.setValue( int( str( v ) ) )

    conf_maxwindowlines = pyqtProperty( "int", lambda x: x.widget.max_window_lines.value(), setMaxwindowlines )
    conf_maxgrowupwindowlines = pyqtProperty( "int", lambda x: x.widget.max_grow_up_window_lines.value(), setMaxgrowupwindowlines )
    conf_windowvariablesthreshold = pyqtProperty( "int", lambda x: x.widget.window_variables_threshold.value(), setWindowvariablesthreshold )
