# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import SIGNAL, pyqtProperty

from core.BaseItem import *

from revkit import circuit, swop, transformation_based_synthesis_func, gate_costs, quantum_costs, transistor_costs

from helpers.RevKitHelper import *

from ui.DesignerWidget import DesignerWidget
from ui.TransformationBasedSynthesis import Ui_TransformationBasedSynthesis

class TransformationBasedSynthesis( DesignerWidget ):
    def __init__( self, parent = None ):
        DesignerWidget.__init__( self, Ui_TransformationBasedSynthesis, parent )

        self.connect( self.swop, SIGNAL( 'currentIndexChanged(int)' ), self.swopChanged )

    def swopChanged( self, index ):
        self.cost_function.setEnabled( index > 0 )
        self.cost_function_label.setEnabled( index > 0 )

class TransformationBasedSynthesisItem( BaseItem ):
    """This item provides the transformation-based synthesis method as well as the corresponding synthesis with output permutation method. The respective synthesis approach can be selected in the pull-down menu (in case of synthesis with output permutation additionally the optimization criteria can be defined). Furthermore, it can be specified whether bi-directional synthesis should be applied or not. After the item has been processed, the enlarged item reports the run-time needed to perform the synthesis."""
    description = "Transformation-based Synthesis"
    custom_size = QSize( 275, 150 )

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )

        self.requires( [ "Truth Table" ] )
        self.provides( [ "Circuit" ] )

    def onCreate( self ):
        self.setText( "Transformation-based" )
        self.setState( self.CONFIGURED )

        self.widget = self.mainWidget( TransformationBasedSynthesis() )

    def executeEvent( self, inputs ):
        circ = circuit()
        cf = [ gate_costs, quantum_costs, transistor_costs ][self.widget.cost_function.currentIndex()]()
        res = swop( circ, inputs[0],
                    enable = self.widget.swop.currentIndex() > 0,
                    exhaustive = self.widget.swop.currentIndex() == 1,
                    synthesis = transformation_based_synthesis_func( bidirectional = self.widget.bidi_synthesis.isChecked() ),
                    cf = cf )
        if type( res ) == dict:
            try:
                circ.circuit_name = inputs[0].name
            except: pass
            self.widget.runtime.setText( "%.2f s" % res['runtime'] )
            circuit_add_runtime( circ, res['runtime'] )
        else:
            pass
        return [ circ ]

    def setBidisynthesis( self, v ):
        self.widget.bidi_synthesis.setChecked( v )

    def setSwop( self, v ):
        self.widget.swop.setCurrentIndex( int( str( v ) ) )

    def setCostfunction( self, v ):
        self.widget.cost_function.setCurrentIndex( int( str( v ) ) )

    conf_bidisynthesis = pyqtProperty( "bool", lambda x: x.widget.bidi_synthesis.isChecked(), setBidisynthesis )
    conf_swop          = pyqtProperty( "int", lambda x: x.widget.swop.currentIndex(), setSwop )
    conf_costfunction  = pyqtProperty( "int", lambda x: x.widget.cost_function.currentIndex(), setCostfunction )
