# RevKit: A Toolkit for Reversible Circuit Design (www.revkit.org)
# Copyright (C) 2009-2011  The RevKit Developers <revkit@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PyQt4.QtCore import QFileInfo
from PyQt4.QtGui import QFileDialog

from core.BaseItem import *
from revkit import write_realization

class WriteRealItem( BaseItem ):
    "This item dumps a given circuit a *.real-file to a given path. A right-click on the item opens a file browser, where the respective file to which the circuit should be stored  can be defined."
    description = "Write Circuit to File"
    iconname = "text-x-readme"

    def __init__( self, parent = None ):
        BaseItem.__init__( self, parent )
        self.requires( [ "Circuit" ] )
        self.filename = QString()

    def onCreate( self ):
        self.setText( "no filename specified" )
        self.actions = [ [ "document-save", "Specify file name", self.setWriteFilename ] ]

    def executeEvent( self, inputs ):
        write_realization( inputs[0], str( self.filename ) )
        return []

    def setWriteFilename( self ):
        filename = QFileDialog.getSaveFileName( None, 'Write Realization', '', 'RevLib Realization (*.real)' )
        self.setFilename( filename )

    def setFilename( self, filename ):
        self.filename = filename
        if not self.filename.isEmpty():
            self.setText( QFileInfo( self.filename ).fileName() )
            self.setState( self.CONFIGURED )

    conf_filename = pyqtProperty( "QString", lambda x: x.filename, setFilename )

